// -----------------------------------------------------------------------------
// Copyright (c) 2014, Nicolas P. Rougier. All Rights Reserved.
// Distributed under the (new) BSD License.
// -----------------------------------------------------------------------------

#include "markers/arrow.glsl"
#include "markers/asterisk.glsl"
#include "markers/chevron.glsl"
#include "markers/clover.glsl"
#include "markers/club.glsl"
#include "markers/cross.glsl"
#include "markers/diamond.glsl"
#include "markers/disc.glsl"
#include "markers/ellipse.glsl"
#include "markers/hbar.glsl"
#include "markers/heart.glsl"
#include "markers/infinity.glsl"
#include "markers/pin.glsl"
#include "markers/ring.glsl"
#include "markers/spade.glsl"
#include "markers/square.glsl"
#include "markers/tag.glsl"
#include "markers/triangle.glsl"
#include "markers/vbar.glsl"
